import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import com.mindfusion.diagramming.Diagram;

public abstract class BaseDialog extends JDialog implements ActionListener, WindowListener {

    private static final long serialVersionUID = 1L;
    static DialogResult result;
    JTable table;
    JTree tree;
    Diagram diagram;


    BaseDialog(Window owner, String title) {
        super(owner, title, ModalityType.APPLICATION_MODAL);
        result = DialogResult.Cancel;

        setType(Type.UTILITY);
        setAlwaysOnTop(true);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setBounds(10, 10, 350, 270);
        setResizable(false);

        JPanel contentPane = new JPanel();
        contentPane.setLayout(null);
        contentPane.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
        setContentPane(contentPane);


        JButton okButton = new JButton("Ok");
        okButton.addActionListener(this);
        okButton.setFocusPainted(false);
        okButton.setActionCommand("ok");
        okButton.setBounds(65, 155, 90, 30);

        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);
        cancelButton.setFocusPainted(false);
        cancelButton.setActionCommand("cancel");
        cancelButton.setBounds(185, 155, 90, 30);


        addWindowListener(this);
        contentPane.add(okButton);
        contentPane.add(cancelButton);

        putWindowInCenter(this);
    }

    private static void putWindowInCenter(Window window) {
        Rectangle winSize = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        double sWidth = screenSize.getWidth();
        double sHeight = screenSize.getHeight();
        Dimension windowSize = window.getSize();

        double wWidth = windowSize.getWidth();
        double wHeight = windowSize.getHeight();
        double x = 0;
        if (wWidth < winSize.width)
            x = (sWidth - wWidth) / 2.0;
        double y = 0;
        if (wHeight < winSize.height)
            y = (sHeight - wHeight) / 2.0;
        window.setLocation((int) x, (int) y);
    }

    @Override
    public abstract void windowClosed(WindowEvent e);
    public abstract void setTable(JTable table);
    public abstract void setTree(JTree tree);

    @Override
    public void windowOpened(WindowEvent e) {

    }

    @Override
    public void windowClosing(WindowEvent e) {

    }

    @Override
    public void windowIconified(WindowEvent e) {

    }

    @Override
    public void windowDeiconified(WindowEvent e) {

    }

    @Override
    public void windowActivated(WindowEvent e) {

    }

    @Override
    public void windowDeactivated(WindowEvent e) {

    }

    void showDialog(String title, String text){
        JOptionPane.showMessageDialog(this, title, text, JOptionPane.INFORMATION_MESSAGE);
    }

}
