import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.*;

//represents the grid with people and contacts at the bottom of the application
public class BaseView extends StackPanel implements ItemListener {

    private static final long serialVersionUID = 1L;
    private JComboBox<Person> dropDown;
    static Map<String, Person[]> comboData;
    static Map<Integer, DefaultCellEditor> editorList;

    BaseView(Window owner) {
        super(owner, new BorderLayout());

        //data with the people
        comboData = new Hashtable<>();
        editorList = new Hashtable<>();

        //create a table model for the table with contact data
        tableModel = new DefaultTableModel() {
            @Override
            public boolean isCellEditable(int row, int column) {
                return column == 2;
            }
        };
        tableModel.addColumn("Person");
        tableModel.addColumn("Phone");
        tableModel.addColumn("Contact");
        tableModel.addColumn("Contact Phone");
        tableModel.addColumn("Contact Date");
        //create a table using the table model
        table = new JTable(tableModel) {
            @Override
            public TableCellEditor getCellEditor(int row, int column) {
                if (column == 2 && editorList.containsKey(row))
                    return editorList.get(row);
                return super.getCellEditor(row, column);
            }
        };
        table.setFillsViewportHeight(true);
        table.getTableHeader().setOpaque(false);
        table.getTableHeader().setBackground(new Color(143, 171, 221));
        table.getTableHeader().setReorderingAllowed(false);
        setUpDropDown(table.getColumnModel().getColumn(2));
        table.getModel().addTableModelListener(e -> {
            if (e.getType() == TableModelEvent.INSERT) {
                dropDown = new JComboBox<>();
                dropDown.addItemListener(BaseView.this);
                if (!editorList.containsKey(e.getLastRow()))
                    editorList.put(e.getLastRow(), new TableListEditor(dropDown));
            }
        });

        JScrollPane pane = new JScrollPane(table);
        pane.setPreferredSize(new Dimension(-1, 200));


        //register our custom ContactNode class with the diagram
        Diagram.registerClass(ContactNode.class, "std:ContactNode", 1, "Mindfusion.Diagramming.ContactNode");


        //create a diagramView to render the diagram
        diagramView = new DiagramView();
        //create a new Diagram and assign it to the view.
        //specify some settings for the diagram
        diagram = new Diagram();
        diagram.setDynamicLinks(true);
        diagram.setLinkHeadShape(Shape.fromId("Arrow"));
        diagram.setLinkShape(LinkShape.Cascading);
        diagram.setLinkSegments(3);
        diagram.setSelectionOnTop(false);
        diagram.setNodesExpandable(false);

        diagramView.setDiagram(diagram);
        diagramView.setAllowInplaceEdit(false);
        diagramView.setBehavior(Behavior.DrawLinks);

        //get the screen dimensions and make the diagram as big as the screen
        Dimension diagramSize = Toolkit.getDefaultToolkit().getScreenSize();
        JScrollPane scrollPane = new JScrollPane(diagramView,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setPreferredSize(new Dimension(diagramSize.width, diagramSize.height));
        scrollPane.setMinimumSize(new Dimension(diagramSize.width, diagramSize.height));

        add(scrollPane, BorderLayout.CENTER);
        add(pane, BorderLayout.SOUTH);

    }

    @Override
    //when a new contact Person is selected we get its data and render it in the table
    public void itemStateChanged(ItemEvent e) {

        if (e.getStateChange() == ItemEvent.SELECTED && table.getSelectedRow() != -1) {
            Person p = (Person) e.getItem();
            if (e.getItemSelectable().equals(editorList.get(table.getSelectedRow()).getComponent())) {
                table.setValueAt(p.getPhone(), table.getSelectedRow(), 3);
                table.setValueAt(p.getContactDate(), table.getSelectedRow(), 4);
            }
        }
    }

    //the method adds a new TableListEditor to the map with editors 
    private void setUpDropDown(TableColumn dropDownColumn) {
        dropDown = new JComboBox<>();
        dropDown.addItemListener(this);

        TableListEditor editor = new TableListEditor(dropDown);

        dropDownColumn.setCellEditor(editor);
        dropDownColumn.setCellRenderer(new DefaultTableCellRenderer());

        editorList.put(0, editor);

    }
    //The cell editor for cells in the column with contacts is a combo box
    private class TableListEditor extends DefaultCellEditor {

        TableListEditor(JComboBox<Person> comboBox) {
            super(comboBox);
        }
    }

}
