
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramLink;

import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import java.awt.*;
import java.awt.event.*;
import java.util.Objects;

//represents the dialog that connects two Person instances
public class ContactDialog extends BaseDialog implements ItemListener {

    private static final long serialVersionUID = 1L;
    protected DatePicker datePicker;
    private JComboBox<String> personOne, personTwo;
    private DiagramLink diagramLink;
    private ContactNode one;
    private ContactNode two;
    private boolean enableBoxes;
    private String date;

    
    /* render the dialog when the contact is created through
     *  drag between two Person nodes
     */
    public ContactDialog(Window owner, Diagram diagram) {
        this(owner, diagram, true, null);

    }

    /* The constructor shows fields that let the user choose contacts. 
     * The constructor is called when the user clicks on the contact icon. */
    public ContactDialog(Window owner, Diagram diagram, boolean enableBoxes, DiagramLink link) {

        super(owner, "Add contact");
        this.diagram = diagram;
        this.enableBoxes = enableBoxes;
        this.diagramLink = link;

        JLabel dateLabel = new JLabel("Date:");
        dateLabel.setBounds(60, 30, 50, 30);
        datePicker = new DatePicker();
        datePicker.setToolTipText("Select date of contact");
        datePicker.setBounds(110, 32, 160, 30);
        getContentPane().add(dateLabel);
        getContentPane().add(datePicker);

        JLabel pOneLabel = new JLabel("Person:");
        pOneLabel.setBounds(60, 65, 50, 30);
        personOne = new JComboBox<>();
        personOne.removeAllItems();
        personOne.setToolTipText("Select person");
        personOne.setBounds(110, 67, 160, 30);
        personOne.addItemListener(this);
        personOne.setEnabled(enableBoxes);
        personOne.setName("one");
        getContentPane().add(pOneLabel);
        getContentPane().add(personOne);

        JLabel pTwoLabel = new JLabel("Person:");
        pTwoLabel.setBounds(60, 100, 50, 30);
        personTwo = new JComboBox<>();
        personTwo.setToolTipText("Select person");
        personTwo.setBounds(110, 102, 160, 30);
        personTwo.addItemListener(this);
        personTwo.setEnabled(enableBoxes);
        personTwo.setName("two");
        getContentPane().add(pTwoLabel);
        getContentPane().add(personTwo);

        personOne.removeAllItems();
        personTwo.removeAllItems();
        personOne.addItem("");
        personTwo.addItem("");
        if (!diagram.getNodes().isEmpty()) {
            for (int i = 0; i < diagram.getNodes().size(); i++) {
                if (diagram.getNodes().get(i) instanceof ContactNode) {
                    personOne.addItem(((ContactNode) diagram.getNodes().get(i)).getName());
                    personTwo.addItem(((ContactNode) diagram.getNodes().get(i)).getName());
                }

            }
        }
        personOne.setSelectedIndex(0);
        personTwo.setSelectedIndex(0);
        if (diagramLink != null) {
            personOne.setSelectedItem(((ContactNode) diagramLink.getOrigin()).getName());
            personTwo.setSelectedItem(((ContactNode) diagramLink.getDestination()).getName());
        }

        setVisible(true);

    }

    //delete a link that represent a contact
    private void deleteItem(DiagramLink item, Diagram diagram) {
        if (!diagram.isItemLocked(item) && diagram.raiseDeleting(item))
            diagram.getItems().remove(item);
    }

    //get the ContactNode with the specified name
    private ContactNode getNode(String name) {

        for (int i = 0; i < diagram.getNodes().size(); i++) {

            if (diagram.getNodes().get(i) instanceof ContactNode && ((ContactNode) diagram.getNodes().get(i)).getName().equals(name)) {
                return (ContactNode) diagram.getNodes().get(i);
            }
        }

        return null;

    }

    //check if contacts can be created
    private boolean canCreate() {

        return diagram != null && diagram.getNodes().size() > 1;
    }

    @Override
    //hndles buttons clicks
    public void actionPerformed(ActionEvent e) {
        String button = e.getActionCommand();
        switch (button) {
            case "ok":
                result = DialogResult.Ok;
                this.dispose();
                break;
            case "cancel":
                result = DialogResult.Cancel;
                this.dispose();
                break;
            default:
                break;
        }
    }

    @Override
    //handles the windowClosed event
    public void windowClosed(WindowEvent e) {
        if (result == DialogResult.Cancel) {
            if (diagramLink != null && diagram != null) {
                deleteItem(diagramLink, diagram);
            }
        }
        //if the OK button was pressed, we create a DigramLink
        if (result == DialogResult.Ok) {

            if (diagramLink != null) {
                updateTable((ContactNode) diagramLink.getOrigin(), (ContactNode) diagramLink.getDestination());
                updateTree((ContactNode) diagramLink.getOrigin(), (ContactNode) diagramLink.getDestination());
            } else {
                if (one != null && two != null) {
                    diagram.getFactory().createDiagramLink(one, two);
                    updateTable(one, two);
                    updateTree(one, two);
                }
            }
        }
    }


    @Override
    //handle the selection of Person instances from the combo boxes of the dialog
    public void itemStateChanged(ItemEvent e) {
        if (enableBoxes) {
            String box = ((JComboBox<String>) e.getSource()).getName();
            switch (box) {
                case "one": //get the first contact Person
                    if (e.getStateChange() == ItemEvent.SELECTED
                            && !Objects.requireNonNull(personOne.getSelectedItem()).toString().isEmpty()) {
                        String firstPerson = personOne.getSelectedItem().toString();
                        if (canCreate())
                            one = getNode(firstPerson);

                    }
                    break;
                case "two": //get the second contact Person
                    if (e.getStateChange() == ItemEvent.SELECTED &&
                            !Objects.requireNonNull(personTwo.getSelectedItem()).toString().isEmpty()) {
                        String secondPerson = personTwo.getSelectedItem().toString();
                        if (canCreate())
                            two = getNode(secondPerson);
                    }
                    break;
                default:
                    break;
            }
        }

    }

    @Override
    //set reference to the table with contacts at the bottom of the app
    public void setTable(JTable table) {
        this.table = table;
    }

    @Override
    //set a reference to the tree with contacts that is rendered to the left of the application
    public void setTree(JTree tree) {
        this.tree = tree;
    }

    //update the table with the newly created Contact data
    private void updateTable(ContactNode origin, ContactNode destination) {
        String org = origin.getName();
        Person[] des = {new Person(destination.getModel())};
        String date = datePicker.getDate();
        des[0].setContactDate(date);
        origin.getModel().getJsonSet().add(des[0].toJson());
        origin.getModel().getContactList().add(des[0]); //changes was made here
        origin.setTag(origin.getModel().getJsonSet());
        if (BaseView.comboData.containsKey(org)) {
            BaseView.comboData.put(org, Utilities.mergeArrays(BaseView.comboData.get(org), des));
        } else {
            BaseView.comboData.put(org, des);
        }
        JComboBox<Person> comboBox = Utilities.getComboBox(org, table);
        int index = Utilities.getRowIndex(org, table);
        comboBox.removeAllItems();
        for (Person s : BaseView.comboData.get(org))
            comboBox.addItem(s);
        if (comboBox.getItemCount() == 1) {
            table.setValueAt(comboBox.getItemAt(0), index, 2);
            table.setValueAt(comboBox.getItemAt(0).getPhone(), index, 3);
            table.setValueAt(date, index, 4);
        }
    }

    //update the tree with the newly created contact data
    private void updateTree(ContactNode origin, ContactNode destination) {
        String personOne = origin.getName();
        String personTwo = destination.getName();
        DefaultMutableTreeNode nodeOne = Utilities.searchNode(personOne, ((DefaultMutableTreeNode) this.tree.getModel().getRoot()));
        DefaultMutableTreeNode nodeTwo = Utilities.searchNode(personTwo, ((DefaultMutableTreeNode) this.tree.getModel().getRoot()));
        assert nodeOne != null;
        assert nodeTwo != null;
        if (((DefaultMutableTreeNode) this.tree.getModel().getRoot()).getChildCount() == 0) {
            ((DefaultMutableTreeNode) this.tree.getModel().getRoot()).add(new DefaultMutableTreeNode(personTwo));
        } else {
            if (nodeOne.isNodeAncestor(nodeTwo)) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) nodeOne.clone();
                DefaultMutableTreeNode s = (DefaultMutableTreeNode) nodeTwo.getParent();
                s.add(node);
                node.add(nodeTwo);
                ((DefaultTreeModel) this.tree.getModel()).reload(s);

            } else {
                 if (nodeTwo.getChildCount() > 0) {
                    if (nodeTwo.getParent().equals(this.tree.getModel().getRoot())) {
                        nodeOne.add(nodeTwo);
                        ((DefaultTreeModel) this.tree.getModel()).reload(((DefaultMutableTreeNode) this.tree.getModel().getRoot()));
                        Utilities.addToParents(nodeOne, nodeTwo, (DefaultTreeModel) this.tree.getModel(), ((DefaultMutableTreeNode) this.tree.getModel().getRoot()));
                    } else {
                        Utilities.cloneTree(nodeOne, nodeTwo, 0);
                        ((DefaultTreeModel) this.tree.getModel()).reload(nodeOne);
                    }

                } else {
                    if (!nodeTwo.getParent().equals(this.tree.getModel().getRoot())) {
                        DefaultMutableTreeNode clone = new DefaultMutableTreeNode(nodeTwo.getUserObject());
                        Utilities.addToSubTrees(nodeOne, clone, (DefaultTreeModel)this.tree.getModel(), ((DefaultMutableTreeNode) this.tree.getModel().getRoot()));
                    } else {
                        nodeOne.add(nodeTwo);
                        ((DefaultTreeModel) this.tree.getModel()).reload(((DefaultMutableTreeNode) this.tree.getModel().getRoot()));
                        Utilities.addToParents(nodeOne, nodeTwo, (DefaultTreeModel) this.tree.getModel(), ((DefaultMutableTreeNode) this.tree.getModel().getRoot()));

                    }

                }

            }
        }
    }
}

