import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.components.*;
import com.mindfusion.diagramming.components.TextComponent;
import com.mindfusion.drawing.*;
import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.awt.*;
import java.io.File;

@SuppressWarnings("WeakerAccess")
public class ContactNode extends CompositeNode {

    private static final Document xmlTemplate = loadXmlTemplate();

    public ContactNode(){

    }

    @SuppressWarnings("unused")
    public ContactNode(Diagram diagram)
    {
        super(diagram);

        getComponents().add(XmlLoader.load(xmlTemplate, null, null));
        findComponents();

        shape.setIsOutline(true);
        setFill(new SolidBrush(Color.WHITE));
        setStroke(new Pen(Color.BLACK, 0.3d));
        setEnabledHandles(AdjustmentHandles.Move);
        image.setImageAlign(ImageAlign.TopCenter);

    }

    public ContactNode(Diagram diagram, Person model){
        super(diagram);
        getComponents().add(XmlLoader.load(xmlTemplate, null, null));
        findComponents();
        shape.setIsOutline(true);
        setImage(model.getImage());
        setName(model.getName());
        setPhone(model.getPhone());
        setBounds(
                (float) model.getPosition().getX(),
                (float) model.getPosition().getY(),
                (float) model.getPosition().getWidth(),
                (float) model.getPosition().getHeight());
        setId(model.getDocumentID());
        setFill(new SolidBrush(Color.WHITE));
        setStroke(new Pen(Color.BLACK, 0.3d));
        setEnabledHandles(AdjustmentHandles.Move);
        image.setImageAlign(ImageAlign.TopCenter);
        this.model = model;

    }

    private static Document loadXmlTemplate() {

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringElementContentWhitespace(true);
        factory.setNamespaceAware(true);

        Document document = null;
        DocumentBuilder builder;
        try
        {
            @SuppressWarnings("SpellCheckingInspection") File file = new File("modelnode.xml");
            if (file.exists()){
                builder = factory.newDocumentBuilder();
                document = builder.parse(file);
            }

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return document;
    }


    private void findComponents(){

        shape = (ShapeComponent) findComponent("Shape");
        image = (ImageComponent) findComponent("Image");
        name = (TextComponent) findComponent("Name");
        phone = (TextComponent) findComponent("Phone");
    }

    @SuppressWarnings("unused")
    public Pen getStroke() {
        return shape.getPen();
    }

    public void setStroke(Pen pen) {
        shape.setPen(pen);
    }
    @SuppressWarnings("unused")
    public Brush getFill()
    {
        return shape.getBrush();
    }

    public void setFill(Brush value)
    {
        shape.setBrush(value);
    }

    public Image getImage()
    {
        return image.getImage();
    }

    public void setImage(Image value)
    {
        image.setImage(value);
    }

    public String getName() {
        return name.getText();
    }

    public void setName(String name) {
        this.name.setText(name);
    }

    public String getPhone() {
        return phone.getText();
    }

    public void setPhone(String phone) {
        this.phone.setText(phone);
    }

    public Person getModel(){
        return this.model;
    }

    public void setModel(Person model) {
        this.model = model;
    }


    @Override
    protected void onLoad(Diagram diagram) {
        super.onLoad(diagram);
        findComponents();
        StackPanel.loadData(diagram.getNodes().size());
    }

    private ShapeComponent shape;
    private ImageComponent image;
    private TextComponent name;
    private TextComponent phone;
    private Person model;
    private static final long serialVersionUID = 1L;
}
