import javax.swing.*;
import java.awt.*;


public final class ContactTracingManager extends JFrame {

    public static void main(String[] args) {

        SwingUtilities.invokeLater(ContactTracingManager::new);

    }


    private ContactTracingManager(){
        setTitle("Contact Tracing Manager");
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setExtendedState(getExtendedState() | MAXIMIZED_BOTH);


        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e) {
            e.printStackTrace();
        }

        JPanel basePanel = new JPanel();
        basePanel.setLayout(new BorderLayout());
        basePanel.setBackground(Color.WHITE);

        BaseView baseView = new BaseView(this);
        TreeView treeView = new TreeView(this);
        ToolBarContent toolBarContent = new ToolBarContent(this, BaseView.diagram, BaseView.table, BaseView.tree);
        MainMenu mainMenu = new MainMenu(BaseView.diagramView, basePanel, this);

        basePanel.add(baseView, BorderLayout.CENTER);
        basePanel.add(treeView, BorderLayout.WEST);
        basePanel.add(toolBarContent, BorderLayout.NORTH);

        add(mainMenu);

        setVisible(true);



    }

}
