import com.mindfusion.diagramming.DiagramView;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.net.URL;
import java.util.Objects;

//Represents the menu strip
class MainMenu extends JPanel {

    private static final long serialVersionUID = 1L;
    private final JPanel gridPanel;
    private JDialog fileDialog;

    MainMenu(DiagramView view, JPanel panel, Window owner) {

        setLayout(new BorderLayout());
        gridPanel = new JPanel();
        gridPanel.setLayout(new GridLayout(0,3,5,5));

        //create the file and guide menu items
        JMenu contact_tracing = new JMenu("File");
        JMenu guide = new JMenu("Guide");

        JMenuItem help = new JMenuItem("Help");

        guide.add(help);
        //create the sub menu items
        JMenuItem save = new JMenuItem("Save", Utilities.getIcon("saveHS.png"));
        JMenuItem load = new JMenuItem("Load");
        JMenuItem exit = new JMenuItem("Exit");

        save.setSize(contact_tracing.getSize());
        load.setSize(contact_tracing.getSize());
        exit.setSize(contact_tracing.getSize());

        contact_tracing.add(save);
        contact_tracing.add(load);
        contact_tracing.add(exit);
        
        //create a menu bar to hold the menu items
        JMenuBar menu = new JMenuBar();
        menu.add(contact_tracing);
        menu.add(guide);
        
        //event handler for pressing the Save button
        save.addActionListener(e -> {
            String op = JOptionPane.showInputDialog(owner, "Enter file name");
            URL url = this.getClass().getResource("files/");
            String path;
            if (op == null || op.isEmpty()){
                path = url.getPath() + "default.json";
            }else if (op.contains(".")){
                String rep = op.substring(0, op.indexOf(".")) + ".json";
                path = url.getPath() + rep;
            }else {
                path = url.getPath() + op +".json";
            }
            view.getDiagram().saveToJsonFile(path);
        });
        //event handler for the load button
        load.addActionListener(e -> {
            File file = new File(this.getClass().getResource("files/").getPath());
            if (Objects.requireNonNull(file.listFiles()).length == 0){
                JOptionPane.showMessageDialog(owner, "No files to load", "",JOptionPane.INFORMATION_MESSAGE);
                return;
            }
            gridPanel.removeAll();
            for (File f : Objects.requireNonNull(file.listFiles())){
                if (f.isFile() && f.getPath().endsWith(".json")){
                    String name = f.getName();
                    FilePanel filePanel = new FilePanel(name, "");
                    gridPanel.add(filePanel);
                    filePanel.getButton().addActionListener(e1 -> {
                        String path = file.getPath() + File.separatorChar + filePanel.getFileToLoad();
                        view.getDiagram().loadFromJsonFile(path);
                        fileDialog.dispose();
                    });

                }
            }
            fileDialog = new JDialog(owner, "Available Files", Dialog.ModalityType.APPLICATION_MODAL);
            fileDialog.setSize(700, 400);
            fileDialog.setResizable(false);
            fileDialog.getContentPane().add(new JScrollPane(gridPanel));
            putWindowInCenter(fileDialog);
            fileDialog.setVisible(true);
        });
        exit.addActionListener(e -> owner.dispose());
        //event handler for the help button. It shows a panel with instructions.
        help.addActionListener(e -> {
            JOptionPane.showMessageDialog(owner, "Press the person icon to create a person or double-click on the diagram area.\n" +
                    "Press the contact icon to create a contact or drag a link between two people.\n" +
                    "Press the layout icons to arrange the nodes in two different layouts.\n" +
                    "The Save menu creates a file with a name of your choice.\n" +
                    "The Load menu lets you load a previously saved diagram.\n" +
                    "The \"Contact\" field in the Contact column in the grid " +
                            "renders a combo box with all people that were in contact with the selected person.\n" +
                            "You can select a person and see their contact data.\n" +
                            "A valid phone number can contain Plus sign (+), WhiteSpace ( ), Hyphen (-), Asterisk (*), and Forward Slash (/)",
                    "", JOptionPane.PLAIN_MESSAGE);
        });

        add(menu, BorderLayout.NORTH);
        add(panel, BorderLayout.CENTER);
    }

    //centers the window in the middle of the screen.
    private static void putWindowInCenter(Window window) {
        Rectangle winSize = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        double sWidth = screenSize.getWidth();
        double sHeight = screenSize.getHeight();
        Dimension windowSize = window.getSize();

        double wWidth = windowSize.getWidth();
        double wHeight = windowSize.getHeight();
        double x = 0;
        if (wWidth < winSize.width)
            x = (sWidth - wWidth) / 2.0;
        double y = 0;
        if (wHeight < winSize.height)
            y = (sHeight - wHeight) / 2.0;
        window.setLocation((int) x, (int) y);
    }
}
