

import com.mindfusion.common.*;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.*;
import java.util.List;

public class Person {

    private String name;
    private String phone;
    private String documentID;
    private BufferedImage image;
    private Rectangle2D position;
    private String contactDate;
    private List<JsonObject> jsonSet;
    private Set<Person> contactList;


    public Person(){
        this("","");

    }
    public Person(String name, String phone){
        this(name, phone, "", null, new Rectangle2D.Float(0,0,0,0));
    }

    public Person(String name, String phone, String documentID, BufferedImage image, Rectangle2D.Float position){
        this.name = name;
        this.phone = phone;
        this.documentID = documentID;
        this.image = validateImage(image);
        this.position = position;
        contactDate = null;
        jsonSet = new ArrayList<>();
        contactList = new HashSet<>();

    }

    public Person(Person prototype){
        this.name = prototype.getName();
        this.phone = prototype.getPhone();
        this.documentID = prototype.documentID;
        this.image = prototype.getImage();
        this.position = prototype.getPosition();
        this.contactDate = prototype.getContactDate();
        this.jsonSet = new ArrayList<>(prototype.getJsonSet());
        this.contactList = new HashSet<>(prototype.getContactList());
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    public String getDocumentID() {
        return documentID;
    }

    public void setDocumentID(String documentID) {
        this.documentID = documentID;
    }

    public BufferedImage getImage() {
        return image;
    }

    public void setImage(BufferedImage image) {
        this.image = validateImage(image);
    }

    public Rectangle2D getPosition() {
        return position;
    }

    public void setPosition(Rectangle2D position) {
        this.position = position;
    }

    public String getContactDate() {
        return contactDate;
    }

    public void setContactDate(String contactDate) {
        this.contactDate = contactDate;
    }


    public Set<Person> getContactList() {
        return contactList;
    }
    public List<JsonObject> getJsonSet(){
        return this.jsonSet;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;
        Person person = (Person) o;
        return this.name.equals(person.name) &&
                this.phone.equals(person.phone);
    }

    @Override
    public int hashCode() {

        int result = 1;
        result = result * 31 + name == null ? 0 : name.hashCode();
        result = result * 31  + phone == null ? 0 : phone.hashCode();
        return result;
    }

    public String toString(){
        return name;
    }

    public Person copy(Person person){
        setName(person.getName());
        setImage(person.getImage());
        setPhone(person.getPhone());
        setDocumentID(person.getDocumentID());
        setContactDate(person.getContactDate());
        setPosition(person.getPosition());
        return this;
    }

    JsonObject toJson(){
        JsonObject jsonObject = new JsonObject();
        jsonObject.put("Name", new JsonValue(this.name));
        jsonObject.put("Phone", new JsonValue(this.phone));
        jsonObject.put("Date", new JsonValue(this.contactDate));
        return jsonObject;
    }

//    public void fromJson(String value) {
//        String data = value.substring(1, value.length()-1);
//        Person per =null;
//        JsonObject jsonObject = null;
//        StringBuilder builder = new StringBuilder();
//        int i;
//        for (i=0; i<data.length(); i++){
//            if (data.charAt(i) == '{'){
//                per = new Person();
//                jsonObject = new JsonObject();
//                continue;
//            }
//            if (data.charAt(i) =='}'){
//                contactList.add(per);
//                jsonSet.add(jsonObject);
//                continue;
//            }
//            if (data.charAt(i)==',')
//                continue;
//            builder.append(data.charAt(i));
//            if (data.charAt(i)=='='){
//                String p = builder.toString().substring(0, builder.toString().length()-1).trim();
//                builder.delete(0, builder.length());
//                i  = readValue(i+1, data, builder);
//                switch (p){
//                    case "Name":
//                        Objects.requireNonNull(per).setName(builder.toString());
//                        jsonObject.put("Name",new JsonValue(builder.toString()));
//                        builder.delete(0, builder.length());
//                        break;
//                    case "Phone":
//                        Objects.requireNonNull(per).setPhone(builder.toString());
//                        jsonObject.put("Phone", new JsonValue(builder.toString()));
//                        builder.delete(0, builder.length());
//                        break;
//                    case "Date":
//                        Objects.requireNonNull(per).setContactDate(builder.toString());
//                        jsonObject.put("Date", new JsonValue(builder.toString()));
//                        builder.delete(0, builder.length());
//                        break;
//                    default:
//                        break;
//                }
//            }
//        }
//
//    }

    public void fromJson(ArrayList<JsonObject> contacts){
        Person person;
        contactList.clear();
        jsonSet.clear();
        for (JsonObject object : contacts){
            person = new Person();
            person.setName(object.get("Name").getValue().toString());
            person.setPhone(object.get("Phone").getValue().toString());
            person.setContactDate(object.get("Date").getValue().toString());
            contactList.add(person);
            jsonSet.add(object);
        }
    }

    private String readKey(int i, String data){
        StringBuilder r = new StringBuilder();
        int c;
        for (c= i; c<data.length(); c++){
            if (data.charAt(c) == ':'){
                break;
            }
            if (data.charAt(c) =='"')
                continue;
            else
                r.append(data.charAt(c));
        }
        i = c;
        return r.toString();
    }
    private int readValue(int i, String data, StringBuilder builder){
        int c;
        for (c = i; c<data.length(); c++){
            if (data.charAt(c) == ',')
                break;
            if (data.charAt(c) == '}'){
                c--;
                break;
            } else{
                builder.append(data.charAt(c));
            }

        }
        return c;
    }


    private BufferedImage validateImage(BufferedImage image){
        try{

            this.image = image == null ? ImageIO.read(this.getClass().getResource("images/image_placeholder.png")) : image;
            this.image = convertToBufferedImage(this.image.getScaledInstance(70,70,BufferedImage.SCALE_SMOOTH));
        }catch (IOException e){
            e.printStackTrace();
        }
        return this.image;
    }

    public static BufferedImage convertToBufferedImage(Image image)
    {
        BufferedImage newImage = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = newImage.createGraphics();
        g.drawImage(image, 0, 0, null);
        g.dispose();
        return newImage;
    }

    public void updateJson(Person oldPerson, Person person) {

        for (JsonObject object : jsonSet){
            if (object.get("Name").getValue().equals(oldPerson.getName()) &&
                    object.get("Phone").getValue().equals(oldPerson.getPhone()))
            {
                object.put("Name", new JsonValue(person.getName()));
                object.put("Phone", new JsonValue(person.getPhone()));
            }
        }
    }

    public void updateContactList(Person oldPerson, Person person) {
        if (contactList.contains(oldPerson)){
            contactList.remove(oldPerson);
            contactList.add(person);
        }
    }
}
