
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramLink;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Arrays;
import java.util.Enumeration;

public class PersonDialog extends BaseDialog {

    private static final long serialVersionUID = 1L;
    private JTextField nameField;
    private JTextField documentID_field;
    private JFormattedTextField phoneNumber;
    private Person person;
    private BufferedImage image;
    private Point2D location;
    private ContactNode editNode;
    private static String lastDir;

    public PersonDialog(Window owner, Point2D location){
        this(owner, location, null);


    }
    public PersonDialog(Window owner, Point2D location, ContactNode editNode){
        super(owner, "Add person");
        this.editNode = editNode;
        person = this.editNode == null ? new Person() : this.editNode.getModel();
        this.location = location;


        JLabel nameLabel = new JLabel("Name:");
        nameLabel.setBounds(60,30,50,20);
        nameField = new JTextField(person.getName());
        nameField.setToolTipText("Person name");
        nameField.setName("name");
        nameField.setBounds(110, 32, 160, 20);
        getContentPane().add(nameLabel);
        getContentPane().add(nameField);

        JLabel documentLabel = new JLabel("ID:");
        documentLabel.setBounds(60, 55, 50,20);
        documentID_field = new JTextField(person.getDocumentID());
        documentID_field.setToolTipText("Person Document ID");
        documentID_field.setName("id");
        documentID_field.setBounds(110, 57, 160, 20);
        getContentPane().add(documentLabel);
        getContentPane().add(documentID_field);

        JLabel phoneLabel = new JLabel("Phone:");
        phoneLabel.setBounds(60, 80, 50,20);
        phoneNumber = new JFormattedTextField(person.getPhone());
        phoneNumber.setToolTipText("Person Phone Number");
        phoneNumber.setName("phone");
        phoneNumber.setBounds(110, 82, 160, 20);
        getContentPane().add(phoneLabel);
        getContentPane().add(phoneNumber);

        JLabel photoLabel = new JLabel("Photo:");
        photoLabel.setBounds(60, 105, 50,20);
        JButton select_photo = new JButton("Import");
        select_photo.addActionListener(this);
        select_photo.setFocusPainted(false);
        select_photo.setActionCommand("photo");
        select_photo.setBounds(110, 107, 160,20);
        getContentPane().add(photoLabel);
        getContentPane().add(select_photo);

        setVisible(true);
    }


    public void setDiagram(Diagram diagram) {
        this.diagram = diagram;
    }

    public Diagram getDiagram(){
        return this.diagram;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        String button = e.getActionCommand();
        switch (button){
            case "ok":
                result = DialogResult.Ok;
                if (dataIsValid())
                    this.dispose();

                break;
            case "cancel":
                result = DialogResult.Cancel;
                this.dispose();
                break;
            case "photo":
                JFileChooser fileChooser;
                if (lastDir != null)
                    fileChooser = new JFileChooser(lastDir);
                else
                    fileChooser = new JFileChooser();
                int result = fileChooser.showOpenDialog(this);
                if (result == JFileChooser.APPROVE_OPTION){
                    File file = fileChooser.getSelectedFile();
                    if (file.exists()){
                        image = Utilities.getImage(file);
                        if (file.getAbsoluteFile().getParent() != null
                                || !file.getAbsoluteFile().getParent().isEmpty())
                            lastDir = file.getAbsoluteFile().getParent();
                    }
                }
                break;

        }
    }

    @Override
    public void windowClosed(WindowEvent e) {
        if (result == DialogResult.Ok){
            if (editNode != null){
                //store old person
                Person oldPerson = new Person().copy(person);
                //update person
                person.setName(nameField.getText());
                person.setDocumentID(documentID_field.getText());
                person.setPhone(phoneNumber.getText());
                if (image != null){
                    person.setImage(image);
                    editNode.setImage(person.getImage());
                }
                //update node
                editNode.setModel(person);
                editNode.setName(person.getName());
                editNode.setId(person.getDocumentID());
                editNode.setPhone(person.getPhone());
                editNode.invalidate();

                //update json data
                for (DiagramLink link : editNode.getIncomingLinks()){
                    Person p = ((ContactNode)link.getOrigin()).getModel();
                    p.updateJson(oldPerson, person);
                    p.updateContactList(oldPerson, person);
                }

                // update table
                DefaultTableModel tableModel = (DefaultTableModel) table.getModel();
                for (int i =0; i<tableModel.getRowCount(); i++){
                    if (tableModel.getValueAt(i, 0).toString().equals(oldPerson.getName())
                            && tableModel.getValueAt(i, 1).toString().equals(oldPerson.getPhone())){
                        tableModel.setValueAt(person.getName(), i, 0);
                        tableModel.setValueAt(person.getPhone(), i, 1);
                    }
                    // update combo box data
                    if (tableModel.getValueAt(i, 2).equals(oldPerson)){
                        Person [] people = BaseView.comboData.get(tableModel.getValueAt(i, 0).toString());
                        Person target = Arrays.stream(people).filter(oldPerson::equals).findAny().orElse(null);
                        if (target != null){
                            target.setName(person.getName());
                            target.setDocumentID(person.getDocumentID());
                            target.setPhone(person.getPhone());
                            target.setImage(person.getImage());
                        }
                        tableModel.setValueAt(person.getName(), i, 2);
                        tableModel.setValueAt(person.getPhone(), i, 3);
                        tableModel.fireTableCellUpdated(i, 2);
                        tableModel.fireTableCellUpdated(i, 3);
                    }else{
                        // update all ather combo boxes
                        Person [] allPeople = BaseView.comboData.get(tableModel.getValueAt(i, 0).toString());
                        if (allPeople != null){
                            Person allTargets = Arrays.stream(allPeople).filter(oldPerson::equals).findAny().orElse(null);
                            if (allTargets != null){
                                allTargets.setName(person.getName());
                                allTargets.setDocumentID(person.getDocumentID());
                                allTargets.setPhone(person.getPhone());
                                allTargets.setImage(person.getImage());
                            }

                        }
                    }

                }
                table.revalidate();

                // update tree
                DefaultMutableTreeNode node = null;
                Enumeration elements = ((DefaultMutableTreeNode)tree.getModel().getRoot()).depthFirstEnumeration();
                while (elements.hasMoreElements()) {
                    node = (DefaultMutableTreeNode) elements.nextElement();
                    if (oldPerson.getName().equals(node.getUserObject().toString())) { //try to use person object insteed of string, more safety
                        node.setUserObject(person.getName());
                        ((DefaultTreeModel)tree.getModel()).reload(node);
                    }
                }


            }else {
                person.setName(nameField.getText());
                person.setDocumentID(documentID_field.getText());
                person.setPhone(phoneNumber.getText());
                person.setImage(image);
                person.setPosition(new Rectangle2D.Float((float)location.getX(), (float)location.getY(), 30,30));
                ContactNode node = new ContactNode(diagram, person);
                diagram.getNodes().add(node);
                ((DefaultTableModel)table.getModel()).addRow(new String[]{person.getName(), person.getPhone(), "","",""});
                ((DefaultMutableTreeNode)tree.getModel().getRoot()).add(new DefaultMutableTreeNode(person.getName()));
                ((DefaultTreeModel)tree.getModel()).reload(((DefaultMutableTreeNode)tree.getModel().getRoot()));
            }


        }
    }

    @Override
    public void setTable(JTable table) {
        this.table = table;
    }

    @Override
    public void setTree(JTree tree) {
        this.tree = tree;
    }

    private boolean dataIsValid(){
        boolean nameIsValid = nameField.getText() != null && !nameField.getText().isEmpty();
        boolean idIsValid = documentID_field.getText() != null && !documentID_field.getText().isEmpty();
        boolean phoneIsValid = Utilities.validatePhone(phoneNumber.getText());
        if (!nameIsValid){
            nameField.setBorder(BorderFactory.createLineBorder(Color.RED));
            showDialog("", "Name is not valid");
            return false;
        }
        if (!idIsValid){
            documentID_field.setBorder(BorderFactory.createLineBorder(Color.RED));
            showDialog("", "ID is not valid");
            return false;
        }
        if (!phoneIsValid){
            phoneNumber.setBorder(BorderFactory.createLineBorder(Color.RED));
            showDialog("",  "Phone number is not valid");
            return false;
        }
        return true;
    }


}
