import com.mindfusion.common.JsonObject;
import com.mindfusion.diagramming.*;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Enumeration;


public class StackPanel extends JPanel {

    private static final long serialVersionUID = 1L;
    static DiagramView diagramView = null;
    static Diagram diagram = null;
    static JTable table = null;
    static JTree tree = null;
    static DefaultMutableTreeNode root;
    static DefaultTreeModel model;
    static DefaultTableModel tableModel;
    private static int count;


    StackPanel(Window owner, LayoutManager2 layout) {
        super();
        setLayout(layout);
        if (diagram != null){

            diagram.addDiagramListener(new DiagramAdapter(){
                @Override
                public void doubleClicked(DiagramEvent diagramEvent) {
                    if (diagramEvent.getMouseButton() == MouseEvent.BUTTON1)
                        ToolBarContent.showPersonDialog(owner, diagramEvent.getPointerPosition());
                }

                @Override
                public void linkCreated(LinkEvent linkEvent) {
                    ToolBarContent.showContactDialog(owner, (Diagram) linkEvent.getSource(), false, linkEvent.getLink());
                }

                @Override
                public void nodeDoubleClicked(NodeEvent nodeEvent) {
                    if (nodeEvent.getMouseButton() == MouseEvent.BUTTON1){
                        ContactNode node = (ContactNode) nodeEvent.getNode();
                        Person person = node.getModel();
                        ToolBarContent.showPersonDialog(owner,  nodeEvent.getPointerPosition(),  node);
                    }


                }

            });
        }

    }


    static void loadData(int size){
        count++;
        boolean isReady = count == size;
        if (isReady){
            count =0;
            root.removeAllChildren();
            tableModel.setRowCount(0);
            BaseView.comboData.clear();

            // load has bugs with date
            for (int i =0; i<diagram.getNodes().size(); i++){
                Person person = new Person();
                ContactNode node = (ContactNode) diagram.getNodes().get(i);
                person.setName(node.getName());
                person.setPhone(node.getPhone());
                person.setDocumentID(node.getId().toString());
                person.setImage((BufferedImage) node.getImage());
                person.setPosition(node.getBounds());
                node.setModel(person);
                if (node.getTag() != null && node.getTag() instanceof ArrayList){
                    person.fromJson((ArrayList<JsonObject>) node.getTag());
                }

                //add to tree
                root.add(new DefaultMutableTreeNode(person.getName()));
                model.reload(root);
                //add to table
                tableModel.addRow(new String[]{person.getName(), person.getPhone(),"","",""});
            }
            for (int i =0; i<diagram.getLinks().size(); i++){
                ContactNode origin = (ContactNode) diagram.getLinks().get(i).getOrigin();
                ContactNode destination = (ContactNode) diagram.getLinks().get(i).getDestination();
                updateTable(origin, destination);
                updateTree(origin, destination);
            }
        }
    }

    private static void updateTable(ContactNode origin, ContactNode destination) {
        String org = origin.getName();
        Person[] des = {new Person(destination.getModel())};
        if (origin.getModel().getContactList().contains(des[0])){
            Person p = origin.getModel().getContactList().stream().filter(des[0]::equals).findAny().orElse(null);
            assert p != null;
            des[0].setContactDate(p.getContactDate());
            origin.getModel().getContactList().remove(des[0]);
            origin.getModel().getContactList().add(des[0]);
        }
        if (BaseView.comboData.containsKey(org)) {
            BaseView.comboData.put(org, Utilities.mergeArrays(BaseView.comboData.get(org), des));
        } else {
            BaseView.comboData.put(org, des);
        }
        JComboBox<Person> comboBox = Utilities.getComboBox(org, table);
        int index = Utilities.getRowIndex(org, table);
        comboBox.removeAllItems();
        for (Person s : BaseView.comboData.get(org))
            comboBox.addItem(s);
        if (comboBox.getItemCount() == 1) {
            table.setValueAt(comboBox.getItemAt(0), index, 2);
            table.setValueAt(comboBox.getItemAt(0).getPhone(), index, 3);
            table.setValueAt(des[0].getContactDate(), index, 4);
        }
    }

    private static void updateTree(ContactNode origin, ContactNode destination) {
        String personOne = origin.getName();
        String personTwo = destination.getName();
        DefaultMutableTreeNode nodeOne = Utilities.searchNode(personOne, root);
        DefaultMutableTreeNode nodeTwo = Utilities.searchNode(personTwo, root);
        assert nodeOne != null;
        assert nodeTwo != null;
        if (root.getChildCount() == 0) {
            root.add(new DefaultMutableTreeNode(personTwo));
        } else {
            if (nodeOne.isNodeAncestor(nodeTwo)) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) nodeOne.clone();
                DefaultMutableTreeNode s = (DefaultMutableTreeNode) nodeTwo.getParent();
                s.add(node);
                node.add(nodeTwo);
                model.reload(s);

            } else {
                //this part need code clean up
                if (nodeTwo.getChildCount() > 0) {
                    if (nodeTwo.getParent().equals(root)) {
                        nodeOne.add(nodeTwo);
                        model.reload(root);
                        Utilities.addToParents(nodeOne, nodeTwo, model, root);
                    } else {
                        Utilities.cloneTree(nodeOne, nodeTwo, 0);
                        model.reload(nodeOne);
                    }

                } else {
                    if (!nodeTwo.getParent().equals(root)) {
                        DefaultMutableTreeNode clone = new DefaultMutableTreeNode(nodeTwo.getUserObject());
                        Utilities.addToSubTrees(nodeOne, clone, model, root);
                    } else {
                        nodeOne.add(nodeTwo);
                        model.reload(root);
                        Utilities.addToParents(nodeOne, nodeTwo, model, root);

                    }

                }

            }
        }
    }

}
