import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramLink;
import com.mindfusion.diagramming.FractalLayout;
import com.mindfusion.diagramming.SpringLayout;

public class ToolBarContent extends JToolBar implements ActionListener {

    JButton addPerson, addContact, fractalLayout, springLayout;
    private static final long serialVersionUID = 1L;
    private Window owner;
    private static Diagram diagram;
    private static JTable table;
    private static JTree tree;


    public ToolBarContent(Window owner, Diagram jDiagram, JTable jTable, JTree jTree){
        this.owner = owner;
        diagram = jDiagram;
        table = jTable;
        tree = jTree;

        addPerson = new JButton();
        addPerson.setIcon(Utilities.getIcon("user_add_48.png"));
        addPerson.setToolTipText("Add new person");
        addPerson.setName("person");
        addPerson.setFocusPainted(false);
        addPerson.addActionListener(this);

        addContact = new JButton();
        addContact.setIcon(Utilities.getIcon("contact_add.png"));
        addContact.setToolTipText("Add new contact");
        addContact.setName("contact");
        addContact.setFocusPainted(false);
        addContact.addActionListener(this);

        fractalLayout = new JButton();
        fractalLayout.setIcon(Utilities.getIcon("fractal_layout1.png"));
        fractalLayout.setToolTipText("Fractal layout");
        fractalLayout.setName("fractal");
        fractalLayout.setFocusPainted(false);
        fractalLayout.addActionListener(this);

        springLayout = new JButton();
        springLayout.setIcon(Utilities.getIcon("spring_layout.png"));
        springLayout.setToolTipText("Spring layout");
        springLayout.setName("spring");
        springLayout.setFocusPainted(false);
        springLayout.addActionListener(this);

        setRollover(true);

        add(addPerson);
        add(addContact);
        add(fractalLayout);
        add(springLayout);

        changeBackgroundColor(this, getComponentCount());


    }


    @Override
    public void actionPerformed(ActionEvent e) {

        String button = ((JButton) e.getSource()).getName();
        switch (button){
            case "person":
                showPersonDialog(this.owner, new Point2D.Float(35,35));
                break;
            case "contact":
                showContactDialog(this.owner, diagram);
                break;
            case "fractal":
                if (containesLinks()){
                    FractalLayout layout = new FractalLayout();
                    layout.setRoot(diagram.getNodes().get(0));
                    layout.arrange(diagram);
                }else{
                    JOptionPane.showMessageDialog(this,
                            "System can not find links in diagram",
                            "Links required", JOptionPane.INFORMATION_MESSAGE);
                }
                break;
            case "spring":
                if (containesLinks()){
                    SpringLayout layout = new SpringLayout();
                    layout.arrange(diagram);
                }else{
                    JOptionPane.showMessageDialog(this,
                            "System can not find links in diagram",
                            "Links required", JOptionPane.INFORMATION_MESSAGE);
                }
                break;
             default:
                 break;
        }
    }

    private void changeBackgroundColor(JComponent content, int size){
        if (content == null)
            return;
        content.setBackground(Color.WHITE);
        changeBackgroundColor((JComponent)getComponentAtIndex(size-1), size-1);
    }

    private boolean containesLinks(){
        return diagram != null && diagram.getLinks().size() > 0;
    }

    public static void showPersonDialog(Window owner, Point2D location) {
        PersonDialog personDialog = new PersonDialog(owner, location);
        personDialog.setDiagram(diagram);
        personDialog.setTable(table);
        personDialog.setTree(tree);
    }
    public static void showPersonDialog(Window owner, Point2D location, ContactNode node){
        PersonDialog personDialog = new PersonDialog(owner, location,  node);
        personDialog.setDiagram(diagram);
        personDialog.setTable(table);
        personDialog.setTree(tree);
    }

    public static void showContactDialog(Window owner, Diagram diagram){
        ContactDialog contactDialog = new ContactDialog(owner, diagram);
        contactDialog.setTable(table);
        contactDialog.setTree(tree);
    }
    public static void showContactDialog(Window owner, Diagram diagram, boolean enableBoxes, DiagramLink link){
        ContactDialog dialog = new ContactDialog(owner, diagram, enableBoxes, link);
        dialog.setTable(table);
        dialog.setTree(tree);
    }
}
