import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Array;
import java.util.Enumeration;
import java.util.regex.Pattern;

public final class Utilities {

    private static final Pattern NUMBER_PATTERN = Pattern.compile("^(?=.*[0-9])[- +*/0-9]+$");


    public static ImageIcon getIcon(String fileName){
        return new ImageIcon(getImage(fileName));
    }

    public static BufferedImage getImage(String fileName){
        BufferedImage img = null;
        try {
            img = ImageIO.read(Utilities.class.getResource("images/"+fileName));
        } catch (IOException e) {
            e.printStackTrace();
        }
        return img;
    }

    public static BufferedImage getImage(File file){
        BufferedImage img = null;
        try {
            img = ImageIO.read(file);
        }catch (IOException e){
            e.printStackTrace();
        }
        return img;
    }
    static Icon resizeIcon(ImageIcon icon, int widht, int height){
        Image img = icon.getImage().getScaledInstance(widht, height, Image.SCALE_SMOOTH);

        return new ImageIcon(img);
    }

    static <T> T[] mergeArrays(T[] a, T[] b) {
        int aLen = a.length;
        int bLen = b.length;

        @SuppressWarnings("unchecked")
        T[] c = (T[]) Array.newInstance(a.getClass().getComponentType(), aLen + bLen);
        System.arraycopy(a, 0, c, 0, aLen);
        System.arraycopy(b, 0, c, aLen, bLen);

        return c;
    }

    public static boolean validatePhone(String text) {
        if (text == null || text.isEmpty())
            return false;
        return NUMBER_PATTERN.matcher(text).matches();
    }

    static int getRowIndex(String rowData, JTable table) {
        for (int i = 0; i < table.getRowCount(); i++)
            if (table.getValueAt(i, 0).equals(rowData))
                return i;
        return -1;
    }
    @SuppressWarnings("unchecked")
    static JComboBox<Person> getComboBox(String rowData, JTable table) {
        JComboBox<Person> box = null;
        for (int i = 0; i < table.getRowCount(); i++) {
            if (table.getValueAt(i, 0).equals(rowData))
                box = (JComboBox) BaseView.editorList.get(i).getComponent();
        }
        return box;
    }
    static void addToParents(DefaultMutableTreeNode nodeOne, DefaultMutableTreeNode nodeTwo, DefaultTreeModel model, DefaultMutableTreeNode root) {
        DefaultMutableTreeNode node;
        Enumeration tree = root.depthFirstEnumeration();
        while (tree.hasMoreElements()) {
            node = (DefaultMutableTreeNode) tree.nextElement();
            if (!nodeOne.equals(node) && node.getUserObject().equals(nodeOne.getUserObject())) {
                cloneTree(node, nodeTwo, 0);
                model.reload(node);
            }
        }
    }
    static void addToSubTrees(DefaultMutableTreeNode nodeOne, DefaultMutableTreeNode clone, DefaultTreeModel model, DefaultMutableTreeNode root) {
        DefaultMutableTreeNode node;
        Enumeration tree = root.depthFirstEnumeration();
        while (tree.hasMoreElements()) {
            node = (DefaultMutableTreeNode) tree.nextElement();
            if (node.equals(nodeOne)) {
                nodeOne.add(clone);
                model.reload(nodeOne);
            } else if (node.getUserObject().equals(nodeOne.getUserObject())) {
                node.add(new DefaultMutableTreeNode(clone.getUserObject()));
                model.reload(node);
            }
        }
    }
    static DefaultMutableTreeNode searchNode(String name, DefaultMutableTreeNode root) {
        DefaultMutableTreeNode node;
        Enumeration tree = root.depthFirstEnumeration();
        while (tree.hasMoreElements()) {
            node = (DefaultMutableTreeNode) tree.nextElement();
            if (name.equals(node.getUserObject().toString())) {
                return node;
            }
        }
        return null;
    }
    static void cloneTree(DefaultMutableTreeNode nodeOne, DefaultMutableTreeNode nodeTwo, int c) {

        if (nodeTwo == null)
            return;
        // add root node of sub tree
        if (c == 0) {
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(nodeTwo.getUserObject());
            nodeOne.add(node);
            nodeOne = node;
        }
        // add all the ather nodes
        for (int i = 0; i < nodeTwo.getChildCount(); i++) {
            DefaultMutableTreeNode child = (DefaultMutableTreeNode) nodeTwo.getChildAt(i);
            DefaultMutableTreeNode clone = new DefaultMutableTreeNode(child.getUserObject());
            nodeOne.add(clone);
            c++;
            cloneTree(clone, child, c);
        }
    }

}
